@@ -38,10 +38,23 @@ where
     fn rotate(self, angle: T) -> Self {
         self.transform(&[
             angle.cos(), angle.sin(), T::zero(),
-            -angle.sin(), angle.cos(), T::zero(),
-            T::zero(), T::zero(), T::one(),
+           -angle.sin(), angle.cos(), T::zero(),
+            T::zero(),   T::zero(),   T::one(),
         ])
     }
+    /// Transform by rotate with a point $\alpha$.
+    ///
+    /// # Matrix
+    ///
+    /// ```math
+    /// \begin{aligned}
+    ///     x =& (x_0 - p_x)\cos α - (y_0 - p_y)\sin α + p_x \\
+    ///     y =& (x_0 - p_x)\sin α + (y_0 - p_y)\cos α + p_y \\
+    /// \end{aligned}
+    /// ```
+    fn rotate_point(self, point: &[T; 2], angle: T) -> Self {
+        self.translate(-point[0], -point[1]).rotate(angle).translate(point[0], point[1])
+    }
     /// Transform by length $\delta x, \delta y$.
     ///
     /// # Matrix
@@ -200,6 +213,12 @@ where
     }
 }
 
+/// 3D projective transformations
+///
+/// # Examples
+#[doc = "```"]
+#[doc = include_str ! ("../../tests/point3d.rs")]
+#[doc = "```"]
 pub trait Projective3D<T>
 where
     Self: Sized,
@@ -218,32 +237,26 @@ where
     /// \end{bmatrix}
     /// ```
     fn transform(self, m: &[T; 16]) -> Self;
-
-    fn rotate(self, a: T, x: T, y: T, z: T) -> Self {
-        self.transform(&[
-            //
-            a.cos() + x * x * (T::one() - a.cos()),
-            x * y * (T::one() - a.cos()) - z * a.sin(),
-            x * z * (T::one() - a.cos()) + y * a.sin(),
-            T::zero(),
-            //
-            y * x * (T::one() - a.cos()) + z * a.sin(),
-            a.cos() + y * y * (T::one() - a.cos()),
-            y * z * (T::one() - a.cos()) - x * a.sin(),
-            T::zero(),
-            //
-            z * x * (T::one() - a.cos()) - y * a.sin(),
-            z * y * (T::one() - a.cos()) + x * a.sin(),
-            T::zero(),
-            T::zero(),
-            //
-            T::zero(),
-            T::zero(),
-            T::zero(),
-            T::one(),
-        ])
+    fn rotate(self, _point: &[T; 3], _alpha: T, _beta: T, _gamma: T) -> Self {
+        todo!()
+    }
+    fn rotate_origin(self, alpha: T, beta: T, gamma: T) -> Self {
+        self.rotate(&[T::zero(), T::zero(), T::zero()], alpha, beta, gamma)
+    }
+    /// Rotate with a line.
+    #[rustfmt::skip]
+    fn rotate_axis(self, _p1: &[T; 3], _p2: &[T; 3], _alpha:T) -> Self {
+         todo!()
+    }
+    fn rotate_axis_x(self, alpha: T) -> Self {
+        self.rotate_axis(&[T::zero(), T::zero(), T::zero()], &[T::one(), T::zero(), T::zero()], alpha)
+    }
+    fn rotate_axis_y(self, alpha: T) -> Self {
+        self.rotate_axis(&[T::zero(), T::zero(), T::zero()], &[T::zero(), T::one(), T::zero()], alpha)
+    }
+    fn rotate_axis_z(self, alpha: T) -> Self {
+        self.rotate_axis(&[T::zero(), T::zero(), T::zero()], &[T::zero(), T::zero(), T::one()], alpha)
     }
-
     #[rustfmt::skip]
     fn translate(self, x: T, y: T, z: T) -> Self {
         self.transform(&[
